package com.ecell.internationalize.app.kafka;

import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Properties;

/**
 * @Title: MyKafkaComfig
 * @Author: liy
 * @Date: 2022/9/2 14:06
 * @Description:
 * @Version:1.0
 */
@Component
public class MyKafkaConfig {
    private static String consumerTopic;
    @Value("${location.topic}")
   public void setConsumerTopic(String topic){
        consumerTopic=topic;
   }
    private static String groupId;
    @Value("${my.groupId}")
    public void setGroupId(String id){
        groupId=id;
    }
    private static String server;
    @Value("${my.server}")
    public void setServer(String serverIp){
        server=serverIp;
    }

    /** 消费者配置信息*/
    public static KafkaConsumer<String,String> inConsumerConfig() {
        Properties properties = new Properties();
        //主机信息
        properties.put("bootstrap.servers",server);
        //群组id
        properties.put("group.id", groupId);
        //消费者是否自动提交偏移量，默认是true
        properties.put("enable.auto.commit", "true");
        //自动提交偏移量的提交频率
        properties.put("auto.commit.interval.ms","1000");
        //latest:在偏移量无效的情况下，消费者将从最新的记录开始读取数据
        // erliest:偏移量无效的情况下，消费者将从起始位置读取分区的记录。
        properties.put("auto.offset.reset", "earliest");
        //消费者在指定的时间内没有发送心跳给群组协调器，就被认为已经死亡，
        properties.put("session.timeout.ms","30000");
        properties.put("key.deserializer","org.apache.kafka.common.serialization.StringDeserializer");
        properties.put("value.deserializer","org.apache.kafka.common.serialization.StringDeserializer");
        KafkaConsumer<String,String> consumer=new KafkaConsumer<>(properties);
        consumer.subscribe(Arrays.asList(consumerTopic.split(",")));
        return consumer;
    }
    /** 生产者配置信息*/
    public static Properties inProducerConfig() {
        Properties kafkaProps=new Properties();
        /**
         * acks指定了必须要有多少个分区副本收到消息，生产者才会认为消息写入成功。
         * acks=0,生产者在写入消息之前不会等待任何来自服务器的响应；就算发送失败了，生产者也不知道。
         * acks=1,只要集群首领收到消息，生产者就会收到一个来自服务器的成功消息
         * acks=all，所有参与复制的节点都收到消息，生产者才会收到一个来自服务器的成功响应。
         */
        kafkaProps.put("acks", "all");
        /**
         * 发送失败后重发的次数，最终还不成功表示发送彻底的失败
         */
        kafkaProps.put("retries", 0);
        /**
         * 默认情况下，消息发送时不会被压缩。
         * snappy:压缩算法由Google发明，它占用较少的CPU,却能提供较好的性能和相当可观的压缩比
         * gzip:占用较多的CPU,但是提供更高的压缩比，带宽比较有限，可以考虑这个压缩算法。
         * 使用压缩可以降低网络传输开销和存储开销，而这往往时向kafka发送消息的瓶颈
         */
        kafkaProps.put("compression.type", "snappy");
        /**
         * 一个批次可以使用的内存大小；当批次被填满，批次里的所有消息会被发送；不过生产者并不一定等批次被填满才发送；
         * 所以批次大小设置得很大，也不会造成延迟，只是会占用更多得内存而已。但是设置得太小，
         * 因为生产者需要更频繁的发送消息，会增加额外的开销。
         */
        kafkaProps.put("batch.size", 100);
        /**
         * 指定了生产者在发送批次之前等待更多消息加入批次的时间。
         * KafkaProducer会在批次填满或liner.ms达到上限时把批次发送出去。
         * 这样做虽然会出现一些延时，但是会提高吞吐量。
         */
        kafkaProps.put("linger.ms", 1);
        /**
         * 生产者内存缓冲区的大小，生产者用它缓冲要发送到服务器的消息。
         * 如果应用程序发送消息的速度超过发送到服务器的速度，会导致生产者空间不足，
         * 这个时候send()方法要么被阻塞，要么抛出异常。
         */
        kafkaProps.put("buffer.memory", 33554432);
        /**
         * 生产者在收到服务器响应之前可以发送多少个消息。
         * 值越高就会占用越多的内存，不过也会提升吞吐量。
         * 设为1可以保证消息是按照发送顺序填写入服务器的，即使发生了重试。
         */
        kafkaProps.put("max.in.flight.requests.per.connection", 1);
        //kafkaProps.put("bootstrap.servers","192.168.123.128:9092,192.168.123.129:9092,192.168.123.130:9092");
        /** 主机信息（broker）*/
        kafkaProps.put("bootstrap.servers",server);
        //键为字符串类型
        kafkaProps.put("key.serializer","org.apache.kafka.common.serialization.StringSerializer");
        //值为字符串类型
        kafkaProps.put("value.serializer","org.apache.kafka.common.serialization.StringSerializer");
        return kafkaProps;
    }
}
