package com.ecell.internationalize.common.system.aspect;

import com.ecell.internationalize.common.core.utils.ServletUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.core.utils.ip.IpUtils;
import com.ecell.internationalize.common.core.utils.locale.LocaleUtil;
import com.ecell.internationalize.common.core.utils.uuid.UUID;
import com.ecell.internationalize.common.security.utils.SecurityUtils;
import com.ecell.internationalize.common.system.annotation.SystemLog;
import com.ecell.internationalize.common.system.constant.FieldConstant;
import com.ecell.internationalize.common.system.entity.SysOperateInfo;
import com.ecell.internationalize.common.system.feign.SysOperatorInfoFeignClient;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

/**
 * @author borui
 */
@Aspect
@Component
public class SystemLogAspect {
    private static final Logger logger = LoggerFactory.getLogger(SystemLogAspect.class);

    @Autowired
    private SysOperatorInfoFeignClient sysOperatorInfoFeignClient;

    /**
     * 处理完请求后执行
     * @param joinPoint 切点
     */
    @AfterReturning(pointcut = "@annotation(controllerLog)", returning = "jsonResult")
    public void doAfterReturning(JoinPoint joinPoint, SystemLog controllerLog, Map<String,Object> jsonResult) {
        handleLog(joinPoint, controllerLog, null, jsonResult);
    }

    /**
     * 拦截异常操作
     *
     * @param joinPoint 切点
     * @param e 异常
     */
    @AfterThrowing(value = "@annotation(controllerLog)", throwing = "e")
    public void doAfterThrowing(JoinPoint joinPoint, SystemLog controllerLog, Exception e) {
        handleLog(joinPoint, controllerLog, e, null);
    }

    protected void handleLog(final JoinPoint joinPoint, SystemLog controllerLog, final Exception e, Map<String,Object> jsonResult) {
        try {
            // ========数据库日志=========*/
            SysOperateInfo sysOperateInfo = new SysOperateInfo();
            String id= UUID.randomUUID().toString(true);
            sysOperateInfo.setId(id);
            String username = SecurityUtils.getUsername();
            if (StringUtils.isNotBlank(username)) {
                sysOperateInfo.setAccount(username);
                sysOperateInfo.setCreateUser(username);
            }
            String ip = IpUtils.getIpAddr(ServletUtils.getRequest());
            System.out.println("获取到的操作机器的IP:"+ip);
            sysOperateInfo.setLoginIp(ip);
            sysOperateInfo.setAccessTime(new Date());
            sysOperateInfo.setStatus(FieldConstant.OPERATOR_SUCCESS);
            if (e != null) {
                sysOperateInfo.setErrorMsg(LocaleUtil.getMessage("messages.system.error"));
                sysOperateInfo.setStatus(FieldConstant.OPERATOR_FAIL);
            }
            else {
                if (FieldConstant.CODE_FIVE == Integer.parseInt(jsonResult.get(FieldConstant.CODE).toString())){
                    //操作失败，记录原因
                    sysOperateInfo.setErrorMsg(jsonResult.get(FieldConstant.MSG).toString());
                    sysOperateInfo.setStatus(FieldConstant.OPERATOR_FAIL);
                }
            }
            // 处理设置注解上的参数
            getControllerMethodDescription(joinPoint, controllerLog, sysOperateInfo, jsonResult);
            // 保存数据库
            sysOperatorInfoFeignClient.save(sysOperateInfo);

        }
        catch (Exception exp) {
            // 记录本地异常日志
            logger.error("==前置通知异常==");
            logger.error("异常信息:{}", exp.getMessage());
            exp.printStackTrace();
        }
    }

    /**
     * 获取注解中对方法的描述信息 用于Controller层注解
     * @param systemLog 日志
     * @param sysOperateInfo 操作日志
     * @throws Exception
     */
    public void getControllerMethodDescription(JoinPoint joinPoint, SystemLog systemLog, SysOperateInfo sysOperateInfo, Map<String,Object> jsonResult) {
        // 设置action动作
        sysOperateInfo.setOperationModule(LocaleUtil.getMessage(systemLog.operation()));
        // 设置标题
        sysOperateInfo.setMsg(LocaleUtil.getMessage(systemLog.msg()));
    }
}
