package com.ecell.internationalize.common.redis.service;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import net.sf.jsqlparser.expression.DateTimeLiteralExpression;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.geo.Circle;
import org.springframework.data.geo.Distance;
import org.springframework.data.geo.GeoResults;
import org.springframework.data.geo.Point;
import org.springframework.data.redis.connection.RedisGeoCommands;
import org.springframework.data.redis.connection.RedisZSetCommands;
import org.springframework.data.redis.core.*;
import org.springframework.stereotype.Component;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author borui
 */
@Component
public class RedisService {
    @Autowired
    public RedisTemplate redisTemplate;


    /**
     * 缓存基本的对象，Integer、String、实体类等
     *
     * @param key   缓存的键值
     * @param value 缓存的值
     */
    public <T> void setCacheObject(final String key, final T value) {
        redisTemplate.opsForValue().set(key, value);
    }

    /**
     * 缓存基本的对象，Integer、String、实体类等
     *
     * @param key      缓存的键值
     * @param value    缓存的值
     * @param timeout  时间
     * @param timeUnit 时间颗粒度
     */
    public <T> void setCacheObject(final String key, final T value, final Long timeout, final TimeUnit timeUnit) {
        redisTemplate.opsForValue().set(key, value, timeout, timeUnit);
    }

    /**
     * 设置有效时间
     *
     * @param key     Redis键
     * @param timeout 超时时间
     * @return true=设置成功；false=设置失败
     */
    public boolean expire(final String key, final long timeout) {
        return expire(key, timeout, TimeUnit.SECONDS);
    }

    /**
     * 设置有效时间
     *
     * @param key     Redis键
     * @param timeout 超时时间
     * @param unit    时间单位
     * @return true=设置成功；false=设置失败
     */
    public boolean expire(final String key, final long timeout, final TimeUnit unit) {
        return redisTemplate.expire(key, timeout, unit);
    }

    /**
     * 获取有效时间
     *
     * @param key Redis键
     * @return 有效时间
     */
    public long getExpire(final String key) {
        return redisTemplate.getExpire(key);
    }

    /**
     * 判断 key是否存在
     *
     * @param key 键
     * @return true 存在 false不存在
     */
    public Boolean hasKey(String key) {
        return redisTemplate.hasKey(key);
    }

    /**
     * 获得缓存的基本对象。
     *
     * @param key 缓存键值
     * @return 缓存键值对应的数据
     */
    public <T> T getCacheObject(final String key) {
        ValueOperations<String, T> operation = redisTemplate.opsForValue();
        return operation.get(key);
    }

    /**
     * 删除单个对象
     *
     * @param key
     */
    public boolean deleteObject(final String key) {
        return redisTemplate.delete(key);
    }

    /**
     * 删除集合对象
     *
     * @param collection 多个对象
     * @return
     */
    public long deleteObject(final Collection collection) {
        return redisTemplate.delete(collection);
    }

    /**
     * 缓存List数据
     *
     * @param key      缓存的键值
     * @param dataList 待缓存的List数据
     * @return 缓存的对象
     */
    public <T> long setCacheList(final String key, final List<T> dataList) {
        Long count = redisTemplate.opsForList().rightPushAll(key, dataList);
        return count == null ? 0 : count;
    }

    /**
     * 获得缓存的list对象
     *
     * @param key 缓存的键值
     * @return 缓存键值对应的数据
     */
    public <T> List<T> getCacheList(final String key) {
        return redisTemplate.opsForList().range(key, 0, -1);
    }

    /**
     * 缓存Set
     *
     * @param key     缓存键值
     * @param dataSet 缓存的数据
     * @return 缓存数据的对象
     */
    public <T> BoundSetOperations<String, T> setCacheSet(final String key, final Set<T> dataSet) {
        BoundSetOperations<String, T> setOperation = redisTemplate.boundSetOps(key);
        Iterator<T> it = dataSet.iterator();
        while (it.hasNext()) {
            setOperation.add(it.next());
        }
        return setOperation;
    }

    /**
     * 获得缓存的set
     *
     * @param key
     * @return
     */
    public <T> Set<T> getCacheSet(final String key) {
        return redisTemplate.opsForSet().members(key);
    }

    /**
     * 缓存Map
     *
     * @param key
     * @param dataMap
     */
    public <T> void setCacheMap(final String key, final Map<String, T> dataMap) {
        if (dataMap != null) {
            redisTemplate.opsForHash().putAll(key, dataMap);
        }
    }

    /**
     * 获得缓存的Map
     *
     * @param key
     * @return
     */
    public <T> Map<String, T> getCacheMap(final String key) {
        return redisTemplate.opsForHash().entries(key);
    }

    /**
     * 往Hash中存入数据
     *
     * @param key   Redis键
     * @param hKey  Hash键
     * @param value 值
     */
    public <T> void setCacheMapValue(final String key, final String hKey, final T value) {
        redisTemplate.opsForHash().put(key, hKey, value);
    }

    /**
     * 获取Hash中的数据
     *
     * @param key  Redis键
     * @param hKey Hash键
     * @return Hash中的对象
     */
    public <T> T getCacheMapValue(final String key, final String hKey) {
        HashOperations<String, String, T> opsForHash = redisTemplate.opsForHash();
        return opsForHash.get(key, hKey);
    }

    /**
     * 获取多个Hash中的数据
     *
     * @param key   Redis键
     * @param hKeys Hash键集合
     * @return Hash对象集合
     */
    public <T> List<T> getMultiCacheMapValue(final String key, final Collection<Object> hKeys) {
        return redisTemplate.opsForHash().multiGet(key, hKeys);
    }

    /**
     * 获得缓存的基本对象列表
     *
     * @param pattern 字符串前缀
     * @return 对象列表
     */
    public Collection<String> keys(final String pattern) {
        return redisTemplate.keys(pattern);
    }

    private final static String GPS_GEO = "GPS_GEO_";
    private final static String GPS_Z_SET = "GPS_Z_SET_";

    /**
     * 存储定位
     * geospatial 地理位置  基本指令    geoAdd    增加一个位置的坐标 经纬度
     *
     * @param key   需要操作地理位置的集合的key
     * @param point 地理位置的对象，x表示经度，y表示维度
     * @param time  存储定位时间
     * @return 返回存储的个数
     */
    public Long geoAdd(String key, Point point, Date time) {
        geoDel(key);
        final long l = time.getTime();//时间排序
        final BoundZSetOperations opsForZSet = redisTemplate.boundZSetOps(GPS_Z_SET + key);
        final Boolean add = opsForZSet.add(l, l);
        if (Boolean.TRUE.equals(add))
            return redisTemplate.opsForGeo().add(GPS_GEO + key, point, l);
        else
            return 0L;
    }

    private void geoDel(String key) {
        /**
         * 删除6个月前面数据,防止redis内存压力大
         */
        final DateTime dateTime = DateUtil.offsetMonth(new DateTime(), -6);
        final long time = dateTime.getTime();
        BoundZSetOperations opsForZSet = redisTemplate.boundZSetOps(GPS_GEO + key);
        final RedisZSetCommands.Range range = new RedisZSetCommands.Range();
        range.lt(time);
        opsForZSet.removeRangeByLex(range);
        opsForZSet = redisTemplate.boundZSetOps(GPS_Z_SET + key);
        opsForZSet.removeRangeByScore(0, time);
    }

    /**
     * 获取定位轨迹
     */
    public List<Point> geoFind(String key, Date start, Date end) {
        final BoundZSetOperations opsForZSet = redisTemplate.boundZSetOps(GPS_Z_SET + key);
        final Set<Long> set = opsForZSet.reverseRangeByScore(start.getTime(), end.getTime());//时间从高到低的排名
        return geoPos(GPS_GEO + key, set);
    }


    /**
     * geospatial 地理位置  基本指令    geoPos    获取多个城市的 经纬度
     *
     * @param key    需要获取的键
     * @param member 需要的获取的地理位置名称
     * @return 返回经纬度集合
     */
    private List<Point> geoPos(String key, Object... member) {
        try {
            return redisTemplate.opsForGeo().position(key, member);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * geospatial 地理位置  基本指令    geoDist    获取两个城市之间的距离
     *
     * @param key     地理位置的集合
     * @param member1 地理位置1
     * @param member2 地理位置2
     * @return 返回两个地理位置之间的距离，默认单位为米
     */
    public Distance geoDist(String key, Object member1, Object member2) {
        key = GPS_GEO + key;
        try {
            return redisTemplate.opsForGeo().distance(key, member1, member2);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * geospatial 地理位置  基本指令    geoRadius    距离地理位置集合中一个地理位置距离范围内的地理位置
     *
     * @param key    地理位置集合的键
     * @param member 地理位置名称
     * @param radius 范围单位 米
     * @return 返回地理位置的集合信息
     */
    public GeoResults<RedisGeoCommands.GeoLocation<Object>> geoRadius(String key, Object member, double radius) {
        key = GPS_GEO + key;
        try {
            return redisTemplate.opsForGeo().radius(key, member, radius);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * geospatial 地理位置  基本指令    geoRadius    距离给定一个经纬度的地理位置距离范围内的地理位置
     *
     * @param key    地理位置集合的键
     * @param circle 给定经纬度和范围的对象 ，需要Point表经纬度和Distinct表示距离
     * @return 返回地理位置的集合信息
     */
    public GeoResults<RedisGeoCommands.GeoLocation<Object>> geoRadius(String key, Circle circle) {
        key = GPS_GEO + key;
        try {
            return redisTemplate.opsForGeo().radius(key, circle);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * geospatial 地理位置  基本指令    geoHash    返回一个位置的hash值，"wx4g0bm9xh0"
     *
     * @param key     需要操作的地理位置集合的键
     * @param members 需要获取hash值的地理位置
     * @return 返回hash值的集合
     */
    public List<String> geoHash(String key, Object... members) {
        key = GPS_GEO + key;
        try {
            return redisTemplate.opsForGeo().hash(key, members);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
