package com.ecell.internationalize.gateway.filter;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ecell.internationalize.common.core.constant.CacheConstants;
import com.ecell.internationalize.common.core.constant.HttpStatus;
import com.ecell.internationalize.common.core.constant.SecurityConstants;
import com.ecell.internationalize.common.core.constant.TokenConstants;
import com.ecell.internationalize.common.core.utils.JwtUtils;
import com.ecell.internationalize.common.core.utils.ServletUtils;
import com.ecell.internationalize.common.core.utils.StringUtils;
import com.ecell.internationalize.common.redis.service.RedisService;
import com.ecell.internationalize.gateway.config.properties.IgnoreWhiteProperties;
import com.ecell.internationalize.gateway.constant.MessagesConstant;
import com.ecell.internationalize.gateway.constant.RateLimitInfo;
import io.jsonwebtoken.Claims;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.Collections;

/**
 * @author borui
 */
@Component
public class AuthFilter implements GlobalFilter, Ordered {
    private static final Logger log = LoggerFactory.getLogger(AuthFilter.class);
    private static final String API_URL = "/yisai-system/api/**";
    private static final String RATE_LIMIT = "rate_limit:";
    private static final String HEADER_APPID = "appid";
    private static final String HEADER_APPKEY = "appkey";

    // 排除过滤的 uri 地址，nacos自行添加
    @Autowired
    private IgnoreWhiteProperties ignoreWhite;

    @Autowired
    private RedisService redisService;

    @Autowired
    RedisScript<Long> redisScript;


    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        ServerHttpRequest.Builder mutate = request.mutate();

        String url = request.getURI().getPath();
        if (StringUtils.isMatch(API_URL, url)) {
            return getVoidMono(exchange, chain, request, url);
            // return chain.filter(exchange);
        }
        // 跳过不需要验证的路径
        if (StringUtils.matches(url, ignoreWhite.getWhites())) {
            return chain.filter(exchange);
        }
        String token = getToken(request);
        if (StringUtils.isEmpty(token)) {

            return unauthorizedResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_TOKE_EMPTY_ZH, MessagesConstant.MESSAGES_TOKE_EMPTY_EN));
        }
        Claims claims = JwtUtils.parseToken(token);
        if (claims == null) {
            return unauthorizedResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_TOKE_EMPTY_OR_ERROR_ZH, MessagesConstant.MESSAGES_TOKE_EMPTY_OR_ERROR_EN));
        }
        String userkey = JwtUtils.getUserKey(claims);
        boolean islogin = redisService.hasKey(getTokenKey(userkey));
        if (!islogin) {
            return unauthorizedResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_LOGIN_STATUS_EXPIRED_ZH, MessagesConstant.MESSAGES_LOGIN_STATUS_EXPIRED_EN));
        }
        String userid = JwtUtils.getUserId(claims);
        String username = JwtUtils.getUserName(claims);
        if (StringUtils.isEmpty(userid) || StringUtils.isEmpty(username)) {
            return unauthorizedResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_TOKE_VERIFICATION_FAILED_ZH, MessagesConstant.MESSAGES_TOKE_VERIFICATION_FAILED_EN));
        }

        // 设置用户信息到请求
        addHeader(mutate, SecurityConstants.USER_KEY, userkey);
        addHeader(mutate, SecurityConstants.DETAILS_USER_ID, userid);
        addHeader(mutate, SecurityConstants.DETAILS_USERNAME, username);
        // 内部请求来源参数清除
        removeHeader(mutate, SecurityConstants.FROM_SOURCE);
        return chain.filter(exchange.mutate().request(mutate.build()).build());
    }


    private void addHeader(ServerHttpRequest.Builder mutate, String name, Object value) {
        if (value == null) {
            return;
        }
        String valueStr = value.toString();
        String valueEncode = ServletUtils.urlEncode(valueStr);
        mutate.header(name, valueEncode);
    }

    private void removeHeader(ServerHttpRequest.Builder mutate, String name) {
        mutate.headers(httpHeaders -> httpHeaders.remove(name)).build();
    }

    private Mono<Void> unauthorizedResponse(ServerWebExchange exchange, String msg) {
        log.error("[鉴权异常处理]请求路径:{}", exchange.getRequest().getPath());

        return ServletUtils.webFluxResponseWriter(exchange.getResponse(), msg, HttpStatus.UNAUTHORIZED);
    }

    private Mono<Void> unauthorizedAPIResponse(ServerWebExchange exchange, String msg, int code) {
        log.error("[对外接口鉴权异常处理]请求路径:{}", exchange.getRequest().getPath());

        return ServletUtils.webFluxResponseWriter(exchange.getResponse(), msg, code);
    }

    /**
     * 获取缓存key
     */
    private String getTokenKey(String token) {
        return CacheConstants.LOGIN_TOKEN_KEY + token;
    }

    /**
     * 获取请求token
     */
    private String getToken(ServerHttpRequest request) {
        String token = request.getHeaders().getFirst(TokenConstants.AUTHENTICATION);
        // 如果前端设置了令牌前缀，则裁剪掉前缀
        if (StringUtils.isNotEmpty(token) && token.startsWith(TokenConstants.PREFIX)) {
            token = token.replaceFirst(TokenConstants.PREFIX, StringUtils.EMPTY);
        }
        return token;
    }

    private Mono<Void> getVoidMono(ServerWebExchange exchange, GatewayFilterChain chain, ServerHttpRequest request, String url) {
        Long apiCount;
        String appId = request.getHeaders().getFirst(HEADER_APPID);
        String appKey = request.getHeaders().getFirst(HEADER_APPKEY);
        if (StringUtils.isEmpty(appId)) {
            return unauthorizedAPIResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_APPID_ISEMPTY_FAILED_ZH, MessagesConstant.MESSAGES_APPID_ISEMPTY_FAILED_EN), MessagesConstant.APP_ID_CODE);
        }
        if (StringUtils.isEmpty(appId)) {
            return unauthorizedAPIResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_APPkEY_ISEMPTY_FAILED_ZH, MessagesConstant.MESSAGES_APPkEY_ISEMPTY_FAILED_EN), MessagesConstant.APP_KEY_CODE);
        }
        String redisKey = appId + "_" + appKey;

        if (!redisService.hasKey(redisKey)) {
            return unauthorizedAPIResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_INTERNAL_SERVER_ERROR_ZH, MessagesConstant.MESSAGES_INTERNAL_SERVER_ERROR_EN), 500);
        }
        try {
            Object cacheObject = redisService.getCacheObject(redisKey);
            String s = JSON.toJSONString(cacheObject);
            RateLimitInfo rateLimitInfo = JSONObject.parseObject(s, RateLimitInfo.class);
            if (StringUtils.isNotNull(rateLimitInfo)) {
                String status = rateLimitInfo.getStatus();
                if (StringUtils.isNotEmpty(status)) {
                    if (MessagesConstant.STRING_ZERO.equals(status)) {
                        return unauthorizedAPIResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_API_STATE_ERROR_ZH, MessagesConstant.MESSAGES_API_STATE_ERROR_EN), MessagesConstant.STATE_CODE);
                    }

                    apiCount = getApiCount(rateLimitInfo, url);
                    if (apiCount > rateLimitInfo.getFrequency()) {
                        return unauthorizedAPIResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_API_LIMIT_MAX_ERROR_ZH, MessagesConstant.MESSAGES_API_LIMIT_MAX_ERROR_EN), MessagesConstant.LIMIT_CODE);
                    }
                }

            }

        } catch (Exception e) {
            return unauthorizedAPIResponse(exchange, MessagesConstant.getMessages(exchange, MessagesConstant.MESSAGES_INTERNAL_SERVER_ERROR_ZH, MessagesConstant.MESSAGES_INTERNAL_SERVER_ERROR_EN), 500);
        }

        return chain.filter(exchange);
    }

    private Long getApiCount(RateLimitInfo info, String url) {
        Long current;
        int time = 0, count = 0;
        String fid;
        StringBuffer buffer = new StringBuffer(RATE_LIMIT);
        if (MessagesConstant.STRING_ONE.equals(info.getFirmFlag())) {
            fid = info.getFirmId();
        } else {
            fid = info.getSecondFirmId();
        }
        String rateKey = buffer.append(fid).append(url).toString();
        time = info.getSecond();
        if (time <= 0) {
            time = 60;
        }
        count = info.getFrequency();
        if (count <= 0) {
            count = 10;
        }

        current = (Long) redisService.redisTemplate.execute(redisScript, Collections.singletonList(rateKey), time, count);

        return current;
    }

    @Override
    public int getOrder() {
        return -200;
    }
}
